//------------------------------------------------------------------------
// This module gets the state transitions from pipe_state_mc and generates
// the appropriate controls for datagen_16bit_top.

// During init_phase: Asserts xmit_TS1. datagen_16bit_top generates TS1 
// initialization sequence.
// Duirng forceElecIdle: Asserts xmit_EIOS. datagen_16bit_top 
// generates EIOS. Placeholder for future use.
// During data_phase: Asserts xmit_data. datagen_16bit_top generates 
// counter data. Currently counter data is not begin used. prbs dats
// is used instead. Placeholder for future use.
//
//------------------------------------------------------------------------

`timescale 1ns/100ps

module datagen_controller ( 
                        inclk,
                        datagen_reset,                       
		        init_phase,
                        data_phase,
//			mgmt_phase,
			input_data_size,
			xmit_TS1
                      
                         );
    
    //inputs                     
    input inclk;
    input datagen_reset;
    input init_phase;
    input data_phase;
//    input mgmt_phase;

    
    //outputs
    output [11:0] input_data_size;
    output xmit_TS1;

    
    //parameters
    parameter DATAPACKET_SIZE = 12'b1111_1111_1111;

    //wires     
    wire [11:0] input_data_size;

    //registers
    reg xmit_TS1;
     
   //Internal Variable declartaions
   reg [3:0] TS1_count = 4'b0000;
 //  reg xmit_data_enable = 1'b0;
 //  reg tmp_cnt = 1'b0; 
	
   //Assignments
   assign input_data_size = DATAPACKET_SIZE;
   
   // Generate controls
   always @(posedge inclk or posedge datagen_reset)
     if(datagen_reset)
     begin
       xmit_TS1              <= 1'b0;
       TS1_count             <= 4'b1111;
     end
     else
     begin
       //SEND TS1 SEQUENCES FOR PCI-E INITIALIZATION; Change the TS1_counter to increase no if TS1 xmit
       if(init_phase == 1'b1)
       begin
                         
         //TS1 condition
         if(TS1_count == 0)
         begin
           xmit_TS1 <= 1'b0;
           TS1_count <= 4'b1111;
         end
         else
         begin
           xmit_TS1 <= 1'b1;
           TS1_count <= TS1_count - 1'b1;
         end
       end
				
       else if(data_phase == 1'b1)  
      	
         xmit_TS1  <= 1'b0;
       else 
        xmit_TS1 <= 1'b0;
    end	

endmodule




