//-----------------------------------------------------------------------------
// Title         : atso_prbs_checker
// Project       : High Speed Elmo
//-----------------------------------------------------------------------------
// File          : atso_prbs_checker.v
// Author        :   <hbutler@SWSJ149>
// Created       : 16.02.2007
// Last modified : 16.02.2007
//-----------------------------------------------------------------------------
// Description :
// This is a 
//-----------------------------------------------------------------------------
// Copyright (c) 2007 by Altera This model is the confidential and
// proprietary property of Altera and the possession or use of this
// file requires a written license from Altera.
//------------------------------------------------------------------------------
// Modification history :
// 16.02.2007 : created
//-----------------------------------------------------------------------------



module prbs_checker_x4 
#(
  parameter NUM_CYCLES_FOR_LOCK = 8'd31,		
  parameter PRBS_INITIAL_VALUE = 97,
  parameter DATA_WIDTH = 8,
  parameter PRBS = 23,
  parameter PIPELINE = 3
  )(
    input  wire [DATA_WIDTH-1:0] dataIn_lane1, 
    input  wire [DATA_WIDTH-1:0] dataIn_lane2, 
    input  wire [DATA_WIDTH-1:0] dataIn_lane3, 
    input  wire [DATA_WIDTH-1:0] dataIn_lane4, 
    output wire lock, 
    output wire errorFlag,
    input  wire clk, 
    input  wire nreset,
    input  wire pause);

  
    wire lock_lane1;
    wire lock_lane2;
    wire lock_lane3;
    wire lock_lane4;
    
    wire errorFlag_lane1;
    wire errorFlag_lane2;
    wire errorFlag_lane3;
    wire errorFlag_lane4;

assign lock = lock_lane1 & lock_lane2 & lock_lane3  & lock_lane4; 
assign errorFlag = errorFlag_lane1 || errorFlag_lane2 || errorFlag_lane3 || errorFlag_lane4; 

prbs_checker 
#(
  .NUM_CYCLES_FOR_LOCK (NUM_CYCLES_FOR_LOCK),		
  .PRBS_INITIAL_VALUE (PRBS_INITIAL_VALUE),
  .DATA_WIDTH (DATA_WIDTH),
  .PRBS (PRBS),
  .PIPELINE (PIPELINE)
  ) prbs_checker_inst_lane1
    (
    .dataIn (dataIn_lane1), 
    .lock (lock_lane1), 
    .errorFlag (errorFlag_lane1),
    .clk (clk), 
    .nreset (nreset),
    .pause (pause)
   );

prbs_checker 
#(
  .NUM_CYCLES_FOR_LOCK (NUM_CYCLES_FOR_LOCK),		
  .PRBS_INITIAL_VALUE (PRBS_INITIAL_VALUE),
  .DATA_WIDTH (DATA_WIDTH),
  .PRBS (PRBS),
  .PIPELINE (PIPELINE)
  ) prbs_checker_inst_lane2
    (
    .dataIn (dataIn_lane2), 
    .lock (lock_lane2), 
    .errorFlag (errorFlag_lane2),
    .clk (clk), 
    .nreset (nreset),
    .pause (pause)
   );

prbs_checker 
#(
  .NUM_CYCLES_FOR_LOCK (NUM_CYCLES_FOR_LOCK),		
  .PRBS_INITIAL_VALUE (PRBS_INITIAL_VALUE),
  .DATA_WIDTH (DATA_WIDTH),
  .PRBS (PRBS),
  .PIPELINE (PIPELINE)
  ) prbs_checker_inst_lane3
    (
    .dataIn (dataIn_lane3), 
    .lock (lock_lane3), 
    .errorFlag (errorFlag_lane3),
    .clk (clk), 
    .nreset (nreset),
    .pause (pause)
   );

prbs_checker 
#(
  .NUM_CYCLES_FOR_LOCK (NUM_CYCLES_FOR_LOCK),		
  .PRBS_INITIAL_VALUE (PRBS_INITIAL_VALUE),
  .DATA_WIDTH (DATA_WIDTH),
  .PRBS (PRBS),
  .PIPELINE (PIPELINE)
  ) prbs_checker_inst_lane4
    (
    .dataIn (dataIn_lane4), 
    .lock (lock_lane4), 
    .errorFlag (errorFlag_lane4),
    .clk (clk), 
    .nreset (nreset),
    .pause (pause)
   );

endmodule
