//----------------------------------------------------------------------
// This module generates 16-bit data for one lane based on the controls 
// received from datagen_controller.v. 
//
// When xmit_TS1 is asserted -> Generates synchronization sequence
// 
//----------------------------------------------------------------------
`timescale 1ns/100ps

module syncgen_16bit_1lane (
 clk,
 reset,
 lane_id,      
 xmit_TS1,  
 data_out, 
 input_data_size      
);       

input clk, reset;
input xmit_TS1;
input[2:0] lane_id;
input[11:0] input_data_size;  //in bytes
output[17:0] data_out; //bit 8 is K char

reg[35:0] TS1_1;
reg[35:0] TS1_2;
reg[35:0] TS1_3;
reg[35:0] TS1_4;

reg[2:0] TS_counter;
reg xmit_TS1_reg;
reg[1:0] byte_counter;

parameter k28_5 = 9'h1bc,
          d21_5 = 9'h0b5,          
          d10_2 = 9'h04a,
          skp   = 9'h11c,
          pad   = 9'h1f7,
          idl   = 9'h17c,
          stp   = 9'h1fb,
          sdp   = 9'h15c,
          endp  = 9'h1fd,
          d5_2  = 9'h045,
          d0_0  = 9'h000,
          d2_0  = 9'h002;
          
reg[17:0] data_out = {k28_5, k28_5};

//loading the memory.
always @ (posedge clk or posedge reset)
begin
 if (reset) begin
   TS1_1 <= {k28_5, skp, skp, skp};   
   TS1_2 <= {k28_5, skp, skp, skp};   
   TS1_3 <= {k28_5, skp, skp, skp};   
   TS1_4 <= {k28_5, skp, skp, skp};   
  end
 else begin
   TS1_1 <=  TS1_1 ;
   TS1_2 <=  TS1_2 ;
   TS1_3 <=  TS1_3 ;
   TS1_4 <=  TS1_4 ;
  end
end

always @ (posedge clk or posedge reset)
 begin
  if (reset)
    xmit_TS1_reg <= 1'b0;
  else if (xmit_TS1)
    xmit_TS1_reg <=   1'b1;
  else if (TS_counter == 3'b011)
   if (byte_counter == 2'b10)
    	xmit_TS1_reg <= 1'b0;
   else
      xmit_TS1_reg <=   1'b1;
  
 end
 
//output appropriate ordered set/counter data 
always @ (posedge clk or posedge reset)
begin
 if (reset) begin
     data_out <= {k28_5, k28_5};    
     TS_counter <= 3'b000;   
     byte_counter <= 2'b00;
 end
 else begin
 TS_counter <= 3'b000; 
    if (xmit_TS1 || xmit_TS1_reg)begin
         if (TS_counter == 3'b000) begin
       	case (byte_counter)      
          2'b00:
          begin 
                data_out <= {TS1_1[26:18], TS1_1[35:27]}; 
                byte_counter <= byte_counter + 2'b10; 
          end
          2'b10:
          begin 
                data_out <= {TS1_1[8:0], TS1_1[17:9]};  
                byte_counter <= byte_counter + 2'b10; 
                TS_counter <= TS_counter + 1'b1;
          end   
	endcase                                         
      end
      
      else if (TS_counter == 3'b001) begin
        case (byte_counter)
          2'b00: 
          begin 
                data_out <= { TS1_2[26:18], TS1_2[35:27]};    
                byte_counter <= byte_counter + 2'b10; 
                TS_counter <= TS_counter; 
          end   
          2'b10: 
          begin  
                data_out <= { TS1_2[8:0], TS1_2[17:9]};      
                byte_counter <= byte_counter + 2'b10;       
                TS_counter <= TS_counter + 1'b1;                      
          end         
        endcase 
      end
      
      else if (TS_counter == 3'b010) begin
        case (byte_counter)
          2'b00:
          begin 
                data_out <= { TS1_3[26:18], TS1_3[35:27]};     
                byte_counter <= byte_counter + 2'b10;
                TS_counter <= TS_counter; 
          end        
          2'b10:
          begin 
                data_out <= { TS1_3[8:0], TS1_3[17:9]};  
                TS_counter <= TS_counter + 1'b1;
                byte_counter <= byte_counter + 2'b10;
          end
        endcase
      end
      
      else if (TS_counter == 3'b011) begin    
        case (byte_counter)
          2'b00: 
          begin 
            data_out <= {TS1_4[26:18], TS1_4[35:27]};  
            byte_counter <= byte_counter + 2'b10;
            TS_counter <= TS_counter; 
          end
          2'b10: 
          begin 
                data_out <= {TS1_4[8:0], TS1_4[17:9]};                                                                     
                TS_counter <= 3'b000; 
                byte_counter <= byte_counter + 2'b10;
           end
        endcase          
      end 
    end
    
   else begin //idle
      case (byte_counter)
        2'b00:   begin data_out <= {k28_5,k28_5}; byte_counter <= byte_counter + 2'b10; end
        default: begin data_out <= {k28_5, k28_5}; byte_counter <= byte_counter + 2'b10; end
      endcase
      TS_counter <= 3'b000;  
    end
  end
end

endmodule
